/*
 *
 *  KDE Blutooth Monitor 
 *
 *  Copyright (C) 2007  Tom Patzig <tpatzig@suse.de>
 *
 *
 *  This file is part of kbluemon.
 *
 *  kbluemon is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  kbluemon is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with kbluemon; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#include "monitor.h"
#include <kdialogbase.h>
#include <qlistview.h>
#include <qdom.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <string>
#include <dbus/dbus.h>



Monitor::Monitor() : MonDialog()
{
	KBluetooth::DBusInit *dbus = new KBluetooth::DBusInit();
	conn = dbus->getDBus();
	manager = new KBluetooth::Manager(conn);
	adapter = new KBluetooth::Adapter(manager->defaultAdapter(), conn);
	
	if (!adapter->isPeriodicDiscovery()) {
		adapter->startPeriodicDiscovery();
		selfstart = 1;
	} else {
		
		connect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
	}
	
	if (!adapter->getPeriodicDiscoveryNameResolving())
		adapter->setPeriodicDiscoveryNameResolving(1);
	

		connect(adapter, SIGNAL(remoteDeviceFound(const QString&, int, short)), this, SLOT(newdev(const QString&, int, short)));
		connect(adapter, SIGNAL(remoteNameUpdated(const QString&, const QString&)), this, SLOT(chg_name(const QString&, const QString&)));
		connect(adapter, SIGNAL(remoteDeviceDisappeared(const QString&)), this, SLOT(deldev(const QString&)));

	
	connect(stopButton, SIGNAL(clicked()), this, SLOT(stopDisco()));
	connect(listView1, SIGNAL(clicked(QListViewItem*)), this, SLOT(chg_details(QListViewItem*)));
	
	rx.setPattern("^\\d+");
	rx.setMinimal(TRUE);

	listView1->hideColumn(3);	
	adjustSize();
	
		
	
}

Monitor::~Monitor() {

	kdDebug() << k_funcinfo << endl;
	if (adapter->isPeriodicDiscovery() && selfstart)
		adapter->stopPeriodicDiscovery();
}




void Monitor::newdev(const QString& name, int dev_class, short rssi) {

	kdDebug() << k_funcinfo << endl;

	rssi = 100.0/(128+20) * (rssi + 128);
	QString msg;
	QString signal = QString::number(rssi) + "%";
	QString devcl = QString::number(dev_class);
	for(it = deviceList.begin(); it!= deviceList.end(); ++it) {
		if ((**it).addr == name) {
			(**it).rssi = rssi;
			QListViewItem *tmp = listView1->findItem((**it).name,0,Qt::ExactMatch);
			tmp->setText(1,signal);
			if ((**it).version == "<i>not available</i>") {
				
				(**it).version = adapter->getRemoteVersion(name);
				(**it).revision = adapter->getRemoteRevision(name);
				(**it).major_class = adapter->getRemoteMajorClass(name);
				(**it).manufacturer = adapter->getRemoteManufacturer(name);

			}
			if (dev_addr->text() == (**it).addr) {
				strength->setProgress(rssi);
				if ((**it).version == "<i>not available</i>")
					chg_details(tmp);	
			}
			
			return;
		}
	}

	RemoteDevice* newDev = getDetails(name,rssi);
	getServices(newDev);
	
	if (deviceList.empty()) {
		strength->setProgress(rssi);
		dev_addr->setText(name);
		dev_version->setText(newDev->version);
		dev_revision->setText(newDev->revision);
		dev_manufacturer->setText(newDev->manufacturer);
		dev_category->setText(QString(newDev->dev_class + ", " + newDev->major_class));
		dev_Info->setText(newDev->devInfo);
		for (newDev->serv_it = newDev->services.begin(); newDev->serv_it!=newDev->services.end(); ++newDev->serv_it) {
			kdDebug() << k_funcinfo << *newDev->serv_it << endl;
			service_list->insertItem(*newDev->serv_it);
		}

	}

	deviceList.push_back(newDev);
	
	QListViewItem *bla = new QListViewItem(listView1, name, signal, newDev->major_class,name);
	listView1->insertItem(bla);
	listView1->hideColumn(3);	
	
}

RemoteDevice* Monitor::getDetails(const QString& addr, short rssi ) {

	QString device_class(adapter->getRemoteMinorClass(addr));
	RemoteDevice *add = new RemoteDevice(addr,rssi,device_class);
	add->name = addr;
	add->version = adapter->getRemoteVersion(addr);
	if (add->version == "")
		add->version = i18n("<i>not available</i>");
	add->manufacturer = adapter->getRemoteManufacturer(addr);
	if (add->manufacturer == "")
		add->manufacturer = i18n("<i>not available</i>");
	add->revision = adapter->getRemoteRevision(addr);
	if (add->revision == "")
		add->revision = i18n("<i>not available</i>");
	add->major_class = adapter->getRemoteMajorClass(addr);
	kdDebug() << "RemoteName: " << adapter->getRemoteName(addr) << endl;
	kdDebug() << "getRemoteClass: " << adapter->getRemoteClass(addr) << endl;
	kdDebug() << "getRemoteFeatures: " << adapter->getRemoteFeatures(addr) << endl;
	kdDebug() << "getRemoteServiceClasses: " << adapter->getRemoteServiceClasses(addr) << endl;

return add;


}
void Monitor::getServices(RemoteDevice* add) {

	KBluetooth::Adapter::ServiceHandleList dev_serves;
	KBluetooth::Adapter::ServiceHandleList::iterator each;
	
	dev_serves = adapter->getRemoteServiceHandles(add->addr, "");

	if (!dev_serves.empty()) {	
	
		for (each = dev_serves.begin(); each!= dev_serves.end(); ++each) {

		
			KBluetooth::ServiceRecord record = adapter->getRemoteServiceRecord(add->addr, *each);
			if ((rx.search( record.name() ) < 0) && (record.name() != "")) 
				add->services.push_back( record.name() );
			
			if ( record.servId().grep("0x1124").size() >= 1) 
				
				add->services.push_back( "Human Interface Device" );

			if (add->devInfo == "" && record.typ() != "")
				add->devInfo = record.typ();
			  

				
		}	
		if (!add->devInfo || add->devInfo == "" )
			add->devInfo = i18n("<i>not available</i>");
	} else {
		add->services.push_back( i18n ("no Services discovered"));
//		kdDebug() << adapter->getError() << endl << endl;;
	}



}


void Monitor::chg_name(const QString& add, const QString& name) {

	kdDebug() << k_funcinfo << endl;
	if (deviceList.empty()) {
		return;
	}
	for(it = deviceList.begin(); it!= deviceList.end(); ++it) {
		if ((**it).getAddr() == add) {
			if ((**it).name == name) {
				return;
			} else {
				if ((**it).getAddr() == dev_addr->text()) {
					dev_name->setText(name);
				}
				(**it).name = name;
				QListViewItem *tmp = listView1->findItem(add,0,Qt::ExactMatch);
				tmp->setText(0,name);
			}
		break;
		}
	}
}


void Monitor::deldev(const QString& add) {
	
	kdDebug() << k_funcinfo << endl;
	for(it = deviceList.begin(); it!= deviceList.end(); ++it) {
	        if ((**it).getAddr() == add) {
			QListViewItem *tmp = listView1->findItem((**it).name,0,Qt::ExactMatch);
			listView1->takeItem(tmp);
			if (deviceList.size() == 1) {
                              deviceList.clear();
                                dev_addr->setText("");
                                dev_name->setText("");
				dev_Info->setText("");
                                strength->setProgress(0);
                                dev_version->setText("");
                                dev_revision->setText("");
                                dev_manufacturer->setText("");
                                dev_category->setText("");
                                service_list->clear();
                        } else {
				if (deviceList.last() == *it) {
					deviceList.pop_back();
                                	*it = deviceList.first();
                                	tmp = listView1->findItem((**it).name,0,Qt::ExactMatch);
				} else {
                                	Monitor::RemoteDeviceList::iterator it2 = deviceList.erase(it);
                                	tmp = listView1->findItem((**it2).name,0,Qt::ExactMatch);
				}
					
	                       	listView1->setSelected(tmp,true);
                                chg_details(tmp);

                        }
		break;
			
		}

	}
}

void Monitor::stopDisco() {
	
	kdDebug() << k_funcinfo << endl;

	
		if (adapter->isPeriodicDiscovery() && selfstart) {
			adapter->stopPeriodicDiscovery();
			selfstart = 0;
			stopButton->setText( i18n("Start Scan"));

		} else if (!adapter->isPeriodicDiscovery()) {
			adapter->startPeriodicDiscovery();
			stopButton->setText( i18n("Stop Scan"));
			selfstart = 1;

		} else if (adapter->isPeriodicDiscovery() && !selfstart && stopButton->text() == "Start Scan") { 
			connect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
			stopButton->setText( i18n("Stop Scan"));
			
			
		} else if (adapter->isPeriodicDiscovery() && !selfstart && stopButton->text() == "Stop Scan") { 

			disconnect(adapter, SIGNAL(remoteDeviceFound(const QString&, int, short)), this, SLOT(newdev(const QString&, int, short)));
			disconnect(adapter, SIGNAL(remoteNameUpdated(const QString&, const QString&)), this, SLOT(chg_name(const QString&, const QString&)));
			disconnect(adapter, SIGNAL(remoteDeviceDisappeared(const QString&)), this, SLOT(deldev(const QString&)));
			disconnect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));

			stopButton->setText( i18n("Start Scan"));
			
		}
	
}

void Monitor::chg_details(QListViewItem* sel) {

	kdDebug() << k_funcinfo << endl;
	if (!sel)
		return;
	
	for(it = deviceList.begin(); it!= deviceList.end(); ++it) {
		if (((**it).name == sel->text(0)) && ((**it).getAddr() == sel->text(3))) {
			
//			kdDebug() << k_funcinfo << *it << endl;
			dev_addr->setText((**it).addr);
			dev_name->setText((**it).name);
			strength->setProgress((**it).rssi);
			dev_version->setText((**it).version);
			dev_revision->setText((**it).revision);
			dev_manufacturer->setText((**it).manufacturer);
			dev_category->setText(QString((**it).dev_class + ", " + (**it).major_class));
			dev_Info->setText((**it).devInfo);
			service_list->clear();
			for ((**it).serv_it = (**it).services.begin(); (**it).serv_it!=(**it).services.end(); ++(**it).serv_it) {
			
				service_list->insertItem(*(**it).serv_it);
			
			}

//		break;
		}

	}
		
}
void Monitor::restartPeriodicDiscovery() {


	kdDebug() << "RestartPeriodicDiscovery" << endl;
	if (!adapter->isPeriodicDiscovery()) {
		adapter->startPeriodicDiscovery();
		selfstart = 1;
		disconnect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
	}
}


#include "monitor.moc"
