/*
 *
 *  Dialogs for kbluetooth Input Devices 
 *
 *  Copyright (C) 2007  Tom Patzig <tpatzig@suse.de>
 *
 *
 *  This file is part of kbluetooth.
 *
 *  kbluetooth is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  kbluetooth is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with kbluetooth; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "inputwizard.h"
#include <kdebug.h>


InputWizard::InputWizard()
{	
	dbus = new DBusInit();
	conn = dbus->getDBus();
	manager = new Manager(conn);

	kdDebug() << k_funcinfo << endl;
	adapter = new Adapter(manager->defaultAdapter(), conn);
		kdDebug() << manager->defaultAdapter() << endl;
	inputdlg = new InputDialog();
	inputdlg_ext = new InputDialog_Ext(inputdlg);
	inputdlg->setExtension(inputdlg_ext->newDevFrame);
	inputdlg->setOrientation(Qt::Vertical);

	inputdlg->show();

	busid = manager->activateService("input");
	inman = new InputManager(conn,busid);

	setupNewDevList = new QStringList();
	inputList = inman->listDevices();

	for (it = inputList.begin(); it != inputList.end(); ++it) {
		kdDebug() << "device path: " << *it << endl;

		newdev = new InputDevice(conn, busid, *it);
		kdDebug() << newdev->getAddress() << endl;
		kdDebug() << adapter->getPath() << endl;
		inputdlg->deviceBox->insertItem(adapter->getRemoteName(newdev->getAddress()));
	}


	connect(inputdlg->addButton, SIGNAL(toggled(bool)), inputdlg, SLOT(showExtension(bool)));
	connect(inputdlg->addButton, SIGNAL(clicked()), SLOT(configNew()));


	connect(inputdlg_ext->setupButton, SIGNAL(clicked()), this, SLOT(setupNewDev()));
	connect(inputdlg->deviceBox, SIGNAL(clicked(QListBoxItem*)), this, SLOT(slotChangeButton(QListBoxItem*)));
	connect(inputdlg->connectButton, SIGNAL(clicked()), this, SLOT(connectdev()));
	connect(inputdlg->okButton, SIGNAL(clicked()), this, SLOT(closeDialog()));
	connect(inputdlg->deleteButton, SIGNAL(clicked()), this, SLOT(deleteDev()));


	
}

InputWizard::~InputWizard() {
	if (adapter->isPeriodicDiscovery() && selfstart)
		adapter->stopPeriodicDiscovery();
	adapter->cancelDiscovery();
	delete inputdlg;

}

void InputWizard::connectdev() {

	kdDebug() << "button:" << inputdlg->connectButton->text() << endl;
	if (inputdlg->connectButton->text() == "d&isconnect") {
		disconnectdev();
		return;
	}

	QListBoxItem *sel = inputdlg->deviceBox->selectedItem();
	QString selName = sel->text();
	inputList = inman->listDevices();
	for (it = inputList.begin();it != inputList.end(); ++it) {
		InputDevice *tmp = new InputDevice(conn,busid,*it);
		QString name = adapter->getRemoteName(tmp->getAddress());
		kdDebug() << "Devicename" << name << endl;
		if (name == selName) { 
			tmp->connectBlock();
			connectMsg(tmp->isConnected(),name);
			return;
		} else 
			continue;
	}

}

void InputWizard::disconnectdev() {

//	if (inputdlg->connectButton->text() == "&connect")
//		return;

	QListBoxItem *sel = inputdlg->deviceBox->selectedItem();
	QString selName = sel->text();
	inputList = inman->listDevices();
	for (it = inputList.begin();it != inputList.end(); ++it) {
		InputDevice *tmp = new InputDevice(conn,busid,*it);
		QString name = adapter->getRemoteName(tmp->getAddress());
		kdDebug() << "Devicename" << name << endl;
		if (name == selName) { 
			tmp->disconnect();
			disconnectMsg(tmp->isConnected(),name); 
			return;
			
		} else 
			continue;
		
	}

}
void InputWizard::deleteDev() {

 	QListBoxItem *sel = inputdlg->deviceBox->selectedItem();
	inputList = inman->listDevices();
//	busid = manager->activateService("input");
	for (it = inputList.begin();it != inputList.end(); ++it) {
		InputDevice *tmp = new InputDevice(conn,busid,*it);
		QString name = adapter->getRemoteName(tmp->getAddress());
		if (name == sel->text()) {
			inman->removeDevice(*it);
			inputdlg->deviceBox->takeItem(sel);
		}
	}
}

void InputWizard::slotChangeButton(QListBoxItem *dev) {
	
	if(dev == NULL)
		return;
	QString name = dev->text();
	inputList = inman->listDevices();
	for (it = inputList.begin();it != inputList.end(); ++it) {
		newdev = new InputDevice(conn, busid, *it);
		kdDebug() << "Devic-connect: " << newdev->isConnected() << endl;
		if (adapter->getRemoteName(newdev->getAddress()) == name) {
			if (newdev->isConnected()) {
				inputdlg->connectButton->setText(i18n("disconnect"));
			} else if (inputdlg->connectButton->text() != "connect") {
				inputdlg->connectButton->setText(i18n("connect"));
			}
		}
	}				
}

void InputWizard::configNew() {
	inputdlg_ext->newdevList->clear();
	if (setupNewDevList->size() > 0) 
		setupNewDevList->clear();
	
	if (inputdlg->addButton->isOn()) {
		inputdlg->addButton->setText(i18n("Add new Device <<"));
		if (!adapter->isPeriodicDiscovery()) { 
		//	adapter->startPeriodicDiscovery();
			adapter->discoverDevices();
			
			selfstart = 1;
		} else {
			connect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
		}
		connect(adapter, SIGNAL(remoteDeviceFound(const QString&, int, short)),this, SLOT(insertDev(const QString&, int, short)));
		connect(adapter, SIGNAL(remoteNameUpdated(const QString&, const QString&)), this, SLOT(updateDevName(const QString&, const QString&)));
	 	
	} else { 
		inputdlg->addButton->setText(i18n("Add new Device >>"));
		adapter->cancelDiscovery();
		if (adapter->isPeriodicDiscovery() && selfstart) {
			adapter->stopPeriodicDiscovery();
			selfstart = 0;
		}
		
		disconnect(adapter, SIGNAL(remoteDeviceFound(const QString&, int, short)),this, SLOT(insertDev(const QString&, int, short)));
		disconnect(adapter, SIGNAL(remoteNameUpdated(const QString&, const QString&)), this, SLOT(updateDevName(const QString&, const QString&)));
		disconnect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
	}
		
	

}

void InputWizard::insertDev(const QString& addr, int /*devclass*/, short /*rssi*/) {
	if (setupNewDevList->grep(addr).size() == 1 )
		return;
	inputList = inman->listDevices();
	for (it = inputList.begin();it != inputList.end(); ++it) {
		newdev = new InputDevice(conn, busid, *it);
		kdDebug() << "Device-connect: " << newdev->isConnected() << endl;
		if (newdev->getAddress() == addr) 
			return;
	}
	QString name = adapter->getRemoteName(addr);
	if (adapter->getRemoteMajorClass(addr) == "peripheral") {
			QListViewItem *newdev = new QListViewItem(inputdlg_ext->newdevList,name, addr);
                        inputdlg_ext->newdevList->insertItem(newdev);
	                setupNewDevList->append(addr);
	} else if (adapter->getRemoteMajorClass(addr) == "audio/video") {
			QListViewItem *newdev = new QListViewItem(inputdlg_ext->newdevList,name, addr);
                      inputdlg_ext->newdevList->insertItem(newdev);
	                setupNewDevList->append(addr);

	} else if(adapter->getRemoteMajorClass(addr) == "computer" )  {
			return;
		
	} else if( adapter->getRemoteMajorClass(addr) == "phone"){

	//	if (inputdlg->deviceBox->findItem(name,Qt::ExactMatch)->text() == name)
	//		return;
		KBluetooth::Adapter::ServiceHandleList dev_serves;
		KBluetooth::Adapter::ServiceHandleList::iterator each;

		dev_serves = adapter->getRemoteServiceHandles(addr, "");

		if (!dev_serves.empty()) {

			for (each = dev_serves.begin(); each!= dev_serves.end(); ++each) {

				KBluetooth::ServiceRecord record = adapter->getRemoteServiceRecord(addr, *each);

			if (record.servId().grep("0x1124").size() >=1) {

				QListViewItem *newdev = new QListViewItem(inputdlg_ext->newdevList,name, addr);
				inputdlg_ext->newdevList->insertItem(newdev);
				setupNewDevList->append(addr);
			}
			}
		}
	}
}


void InputWizard::updateDevName(const QString& addr, const QString& newname) {
        if (setupNewDevList->isEmpty() || setupNewDevList->grep(addr).size() < 1)
                        return;
        QListViewItem *tmp = inputdlg_ext->newdevList->findItem(addr,1,Qt::ExactMatch);

        if (tmp->text(0) == "") {
                inputdlg_ext->newdevList->takeItem(tmp);
                QListViewItem *newdev = new QListViewItem(inputdlg_ext->newdevList,newname,addr);
                inputdlg_ext->newdevList->insertItem(newdev);
        }
}



void InputWizard::closeDialog() {
	if (adapter->isPeriodicDiscovery() && selfstart)
		adapter->stopPeriodicDiscovery();
	adapter->cancelDiscovery();
	inputdlg->close();
	KApplication::kApplication()->quit();
}


void InputWizard::setupNewDev() {

	QListViewItem *confSel = inputdlg_ext->newdevList->currentItem();
	SelectAddr = confSel->text(1);
	kdDebug() << "address: _____________" << SelectAddr << endl; 
	adapter->stopPeriodicDiscovery();
	adapter->cancelDiscovery();

	if (adapter->getRemoteMinorClass(SelectAddr) == "keyboard") {
		if (adapter->hasBonding(SelectAddr)) {
			adapter->removeBonding(SelectAddr);
		}
		adapter->createBondingBlock(SelectAddr);
	}
//	connect(inman, SIGNAL(deviceCreated(const QString&)), this, SLOT(setupNewDevCallback(const QString&)));
	
	QString remoteConn = inman->createDeviceBlock(SelectAddr);

	if (remoteConn == "/org/bluez/input") {
	
		QMessageBox::information(inputdlg, i18n("Creating Input Device failed"), i18n("Failed to create the input device for %1!").arg(adapter->getRemoteName(SelectAddr)), QMessageBox::Ok|QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);

	}
	
	inputdlg->deviceBox->insertItem(confSel->text(0));
	inputdlg_ext->newdevList->takeItem(confSel);
	inputdlg->addButton->setOn(0);

	configNew();

	setupNewDevCallback(remoteConn);

}

void InputWizard::setupNewDevCallback(const QString &remoteConn) {

	InputDevice *tmp;

	kdDebug() << "REMOTECONNECTION:  " << remoteConn << endl;
        if (!remoteConn.isNull() ) {

//		busid = manager->activateService("input");
                tmp = new InputDevice(conn,busid,remoteConn);

        } else {
	        kdDebug() << "LISTDEVICE:  " << inman->listDevices() << endl;
        	inputList = inman->listDevices();
        	for (it = inputList.begin();it != inputList.end(); ++it) {
//			busid = manager->activateService("input");
                	tmp = new InputDevice(conn,busid,*it);
                	if (tmp->getAddress() != SelectAddr)
                        	continue;
		}
	
	}
	int connectit = QMessageBox::question( inputdlg, i18n("Connect now?"),
	i18n("Would you like to connect to %1 now?").arg(adapter->getRemoteName(SelectAddr)),
	QMessageBox::Yes|QMessageBox::Default,
	QMessageBox::No|QMessageBox::Escape,
	QMessageBox::NoButton);
	if (connectit == QMessageBox::Yes) { 
//	       	bool connectnow = tmp->connectBlock();
	       	tmp->connect();
		connect(tmp, SIGNAL(connected()), this, SLOT(connectMsg(bool, QString)));
//		connectMsg(connectnow,adapter->getRemoteName(SelectAddr));
	}
        kdDebug() << "SETUP---READY:  " << adapter->getRemoteName(SelectAddr) << endl;
}


void InputWizard::restartPeriodicDiscovery() {


	kdDebug() << "RestartPeriodicDiscovery" << endl;
	if (!adapter->isPeriodicDiscovery()) {
		adapter->startPeriodicDiscovery();
		selfstart = 1;
		disconnect(adapter, SIGNAL(periodicDiscoveryStopped()), this, SLOT(restartPeriodicDiscovery()));
	}
}
	
void InputWizard::connectMsg(bool connect, QString name) {

	if (connect) {
		QMessageBox::information(inputdlg, i18n("Connection succesfull"), i18n("Connection to %1 successful established!").arg(name), QMessageBox::Ok|QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		inputdlg->connectButton->setText(i18n("disconnect"));
		
//		disconnect(tmp, SIGNAL(connected()), this, SLOT(connectMsg(bool, QString)));
	} else {
		QMessageBox::information(inputdlg, i18n("Connection failed"), i18n("Could not connect to ") + name + i18n("\n Check if the device is available and bluetooth enabled."), QMessageBox::Ok|QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		inputdlg->connectButton->setText(i18n("connect"));

	}
}

void InputWizard::disconnectMsg(bool connect, QString name) {
	if (!connect) {
	QMessageBox::information(inputdlg, i18n("Disconnect succesfull"), i18n("Connection to ") + name + i18n(" successful disconnected!"), QMessageBox::Ok|QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
			inputdlg->connectButton->setText(i18n("connect"));
	} else {
		QMessageBox::information(inputdlg, i18n("Disconnect failed"), i18n("Connection to ") + name + i18n(" could not be disconnected!"), QMessageBox::Ok|QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton);
		inputdlg->connectButton->setText(i18n("disconnect"));
	}
}

#include "inputwizard.moc"

